@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -16,8 +16,16 @@
 optionlist dnslookup_router_options[] = {
   { "check_secondary_mx", opt_bool,
       (void *)(offsetof(dnslookup_router_options_block, check_secondary_mx)) },
+  { "check_srv",          opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, check_srv)) },
+  { "dnssec_request_domains",         opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, dnssec_request_domains)) },
+  { "dnssec_require_domains",         opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, dnssec_require_domains)) },
   { "mx_domains",         opt_stringptr,
       (void *)(offsetof(dnslookup_router_options_block, mx_domains)) },
+  { "mx_fail_domains",    opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, mx_fail_domains)) },
   { "qualify_single",     opt_bool,
       (void *)(offsetof(dnslookup_router_options_block, qualify_single)) },
   { "rewrite_headers",    opt_bool,
@@ -26,6 +34,8 @@ optionlist dnslookup_router_options[] = {
       (void *)(offsetof(router_instance, same_domain_copy_routing)) },
   { "search_parents",     opt_bool,
       (void *)(offsetof(dnslookup_router_options_block, search_parents)) },
+  { "srv_fail_domains",   opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, srv_fail_domains)) },
   { "widen_domains",      opt_stringptr,
       (void *)(offsetof(dnslookup_router_options_block, widen_domains)) }
 };
@@ -44,7 +54,12 @@ dnslookup_router_options_block dnslookup_router_option_defaults = {
   FALSE,           /* search_parents */
   TRUE,            /* rewrite_headers */
   NULL,            /* widen_domains */
-  NULL             /* mx_domains */
+  NULL,            /* mx_domains */
+  NULL,            /* mx_fail_domains */
+  NULL,            /* srv_fail_domains */
+  NULL,            /* check_srv */
+  NULL,            /* dnssec_request_domains */
+  NULL             /* dnssec_require_domains */
 };
 
 
@@ -77,6 +92,7 @@ rblock = rblock;
 DECLINE
   . the domain does not exist in the DNS
   . MX records point to non-existent hosts (including RHS = IP address)
+  . a single SRV record has a host name of "." (=> no service)
   . syntactically invalid mail domain
   . check_secondary_mx set, and local host not in host list
 
@@ -116,7 +132,7 @@ dnslookup_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   struct passwd *pw,              /* passwd entry after check_local_user */
-  BOOL verify,                    /* TRUE when verifying */
+  int verify,                     /* v_none/v_recipient/v_sender/v_expn */
   address_item **addr_local,      /* add it to this if it's local */
   address_item **addr_remote,     /* add it to this if it's remote */
   address_item **addr_new,        /* put new addresses on here */
@@ -125,8 +141,10 @@ dnslookup_router_entry(
 host_item h;
 int rc;
 int widen_sep = 0;
+int whichrrs = HOST_FIND_BY_MX | HOST_FIND_BY_A;
 dnslookup_router_options_block *ob =
   (dnslookup_router_options_block *)(rblock->options_block);
+uschar *srv_service = NULL;
 uschar *widen = NULL;
 uschar *pre_widen = addr->domain;
 uschar *post_widen = NULL;
@@ -141,6 +159,20 @@ DEBUG(D_route)
   debug_printf("%s router called for %s\n  domain = %s\n",
     rblock->name, addr->address, addr->domain);
 
+/* If an SRV check is required, expand the service name */
+
+if (ob->check_srv != NULL)
+  {
+  srv_service = expand_string(ob->check_srv);
+  if (srv_service == NULL && !expand_string_forcedfail)
+    {
+    addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
+      rblock->name, ob->check_srv, expand_string_message);
+    return DEFER;
+    }
+  else whichrrs |= HOST_FIND_BY_SRV;
+  }
+
 /* Set up the first of any widening domains. The code further down copes with
 either pre- or post-widening, but at present there is no way to turn on
 pre-widening, as actually doing so seems like a rather bad idea, and nobody has
@@ -149,9 +181,22 @@ precedence over global names. For example, if the domain is "xxx.ch" it might
 be something in the "ch" toplevel domain, but it also might be xxx.ch.xyz.com.
 The choice of pre- or post-widening affects which takes precedence. If ever
 somebody comes up with some kind of requirement for pre-widening, presumably
-with some conditions under which it is done, it can be selected here. */
+with some conditions under which it is done, it can be selected here.
 
-if (ob->widen_domains != NULL)
+The rewrite_headers option works only when routing an address at transport
+time, because the alterations to the headers are not persistent so must be
+worked out immediately before they are used. Sender addresses are routed for
+verification purposes, but never at transport time, so any header changes that
+you might expect as a result of sender domain widening do not occur. Therefore
+we do not perform widening when verifying sender addresses; however, widening
+sender addresses is OK if we do not have to rewrite the headers. A corollary
+of this is that if the current address is not the original address, then it
+does not appear in the message header so it is also OK to widen. The
+suppression of widening for sender addresses is silent because it is the
+normal desirable behaviour. */
+
+if (ob->widen_domains != NULL &&
+    (verify != v_sender || !ob->rewrite_headers || addr->parent != NULL))
   {
   listptr = ob->widen_domains;
   widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
@@ -172,6 +217,7 @@ decision as to which is taken above. */
 
 for (;;)
   {
+  int flags = whichrrs;
   BOOL removed = FALSE;
 
   if (pre_widen != NULL)
@@ -203,6 +249,7 @@ for (;;)
 
   h.next = NULL;
   h.address = NULL;
+  h.port = PORT_NONE;
   h.mx = MX_NONE;
   h.status = hstatus_unknown;
   h.why = hwhy_unknown;
@@ -210,15 +257,24 @@ for (;;)
 
   /* Unfortunately, we cannot set the mx_only option in advance, because the
   DNS lookup may extend an unqualified name. Therefore, we must do the test
-  subsequently. */
+  subsequently. We use the same logic as that for widen_domains above to avoid
+  requesting a header rewrite that cannot work. */
 
-  rc = host_find_bydns(&h, rblock->ignore_target_hosts, FALSE, FALSE,
-    ob->qualify_single, ob->search_parents, &fully_qualified_name,
-    &removed);
+  if (verify != v_sender || !ob->rewrite_headers || addr->parent != NULL)
+    {
+    if (ob->qualify_single) flags |= HOST_FIND_QUALIFY_SINGLE;
+    if (ob->search_parents) flags |= HOST_FIND_SEARCH_PARENTS;
+    }
+
+  rc = host_find_bydns(&h, rblock->ignore_target_hosts, flags, srv_service,
+    ob->srv_fail_domains, ob->mx_fail_domains,
+    ob->dnssec_request_domains, ob->dnssec_require_domains,
+    &fully_qualified_name, &removed);
   if (removed) setflag(addr, af_local_host_removed);
 
   /* If host found with only address records, test for the domain's being in
-  the mx_domains list. */
+  the mx_domains list. Note that this applies also to SRV records; the name of
+  the option is historical. */
 
   if ((rc == HOST_FOUND || rc == HOST_FOUND_LOCAL) && h.mx < 0 &&
        ob->mx_domains != NULL)
@@ -254,27 +310,34 @@ for (;;)
 
   if (rc != HOST_FIND_FAILED) break;
 
-  /* Check to see if the failure is the result of MX records pointing
-  to non-existent domains, and if so, set an appropriate error message;
-  otherwise "unknown mail domain" is used, which is confusing. Also, in this
-  case don't do the widening. We need check only the first host to see if
-  its MX has been filled in, but there is no address, because if there were
-  any usable addresses returned, we would not have had HOST_FIND_FAILED.
+  /* Check to see if the failure is the result of MX records pointing to
+  non-existent domains, and if so, set an appropriate error message; the case
+  of an MX or SRV record pointing to "." is another special case that we can
+  detect. Otherwise "unknown mail domain" is used, which is confusing. Also, in
+  this case don't do the widening. We need check only the first host to see if
+  its MX has been filled in, but there is no address, because if there were any
+  usable addresses returned, we would not have had HOST_FIND_FAILED.
 
   As a common cause of this problem is MX records with IP addresses on the
   RHS, give a special message in this case. */
 
   if (h.mx >= 0 && h.address == NULL)
     {
-    addr->message = US"all relevant MX records point to non-existent hosts";
-    if (!allow_mx_to_ip && string_is_ip_address(h.name, NULL))
+    setflag(addr, af_pass_message);   /* This is not a security risk */
+    if (h.name[0] == 0)
+      addr->message = US"an MX or SRV record indicated no SMTP service";
+    else
       {
-      addr->user_message =
-        string_sprintf("It appears that the DNS operator for %s\n"
-          "has installed an invalid MX record with an IP address\n"
-          "instead of a domain name on the right hand side.", addr->domain);
-      addr->message = string_sprintf("%s or (invalidly) to IP addresses",
-        addr->message);
+      addr->message = US"all relevant MX records point to non-existent hosts";
+      if (!allow_mx_to_ip && string_is_ip_address(h.name, NULL) != 0)
+        {
+        addr->user_message =
+          string_sprintf("It appears that the DNS operator for %s\n"
+            "has installed an invalid MX record with an IP address\n"
+            "instead of a domain name on the right hand side.", addr->domain);
+        addr->message = string_sprintf("%s or (invalidly) to IP addresses",
+          addr->message);
+        }
       }
     return DECLINE;
     }
